==> whitemagic-2.1.7.tar.gz: whitemagic-2.1.7/tests/test_api_recent_fixes.py <==
"""
Integration tests for recently fixed API endpoints.

These tests verify the fixes from the third independent review:
1. consolidate endpoint uses consolidate_short_term (not consolidate_memories)
2. stats endpoint builds stats from list_all_memories and list_all_tags
3. tags endpoint uses list_all_tags (not list_tags)
"""

import pytest
import pytest_asyncio
from fastapi.testclient import TestClient
from httpx import AsyncClient, ASGITransport
from whitemagic.api.app import app
from whitemagic.api.database import Database, User, APIKey
from whitemagic.api.auth import generate_api_key
from whitemagic.api.dependencies import set_database
from sqlalchemy.ext.asyncio import AsyncSession
import os
import tempfile
import shutil
from pathlib import Path


@pytest_asyncio.fixture
async def test_db():
    """Create a test database."""
    db = Database("sqlite+aiosqlite:///:memory:")
    await db.create_tables()
    set_database(db)  # Register with dependency injection
    yield db
    await db.close()


@pytest_asyncio.fixture
async def test_user(test_db):
    """Create a test user with API key."""
    async with test_db.async_session() as session:
        # Create user
        user = User(email="test@example.com", plan_tier="pro", whop_user_id="test_user_123")
        session.add(user)
        await session.commit()
        await session.refresh(user)

        # Generate API key
        full_key, key_hash, key_prefix = generate_api_key("prod")

        api_key = APIKey(
            user_id=user.id,
            key_hash=key_hash,
            key_prefix=key_prefix,
            is_active=True,
        )
        session.add(api_key)
        await session.commit()

        yield user, full_key


@pytest_asyncio.fixture
async def auth_headers(test_user):
    """Create authorization headers."""
    _, api_key = test_user
    return {"Authorization": f"Bearer {api_key}"}


@pytest_asyncio.fixture
async def memory_dir():
    """Create a temporary directory for test memories."""
    temp_dir = tempfile.mkdtemp()

    # Create memory structure
    memory_path = Path(temp_dir) / "memory"
    (memory_path / "short_term").mkdir(parents=True, exist_ok=True)
    (memory_path / "long_term").mkdir(parents=True, exist_ok=True)
    (memory_path / "archive").mkdir(parents=True, exist_ok=True)

    # Set environment variable
    old_base = os.environ.get("WM_BASE_PATH")
    os.environ["WM_BASE_PATH"] = temp_dir

    yield temp_dir

    # Cleanup
    if old_base:
        os.environ["WM_BASE_PATH"] = old_base
    else:
        os.environ.pop("WM_BASE_PATH", None)

    shutil.rmtree(temp_dir, ignore_errors=True)


@pytest.mark.asyncio
async def test_consolidate_endpoint_uses_correct_method(test_db, auth_headers, memory_dir):
    """
    Verify consolidate endpoint calls consolidate_short_term.

    This was the primary bug: endpoint called manager.consolidate_memories
    which doesn't exist. Should call manager.consolidate_short_term.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        response = await client.post(
            "/api/v1/consolidate", json={"dry_run": True, "min_age_days": 30}, headers=auth_headers
        )

        # Should not raise AttributeError for missing method
        assert response.status_code == 200
        data = response.json()

        # Verify response structure
        assert "success" in data
        assert "archived_count" in data
        assert "promoted_count" in data
        assert "dry_run" in data
        assert data["dry_run"] is True


@pytest.mark.asyncio
async def test_stats_endpoint_format(test_db, auth_headers, memory_dir):
    """
    Verify stats endpoint returns correct format.

    Bug fix: endpoint called manager.get_stats which doesn't exist.
    Now builds stats from list_all_memories and list_all_tags.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        response = await client.get("/api/v1/stats", headers=auth_headers)

        assert response.status_code == 200
        data = response.json()

        # Verify required fields
        assert "success" in data
        assert "short_term_count" in data
        assert "long_term_count" in data
        assert "total_count" in data
        assert "total_tags" in data
        assert "most_used_tags" in data

        # Verify types
        assert isinstance(data["short_term_count"], int)
        assert isinstance(data["long_term_count"], int)
        assert isinstance(data["total_count"], int)
        assert isinstance(data["total_tags"], int)
        assert isinstance(data["most_used_tags"], list)


@pytest.mark.asyncio
async def test_stats_most_used_tags_format(test_db, auth_headers, memory_dir):
    """
    Verify most_used_tags is properly formatted as list of [tag, count] tuples.

    This ensures the fix properly extracts tag data from list_all_tags response.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        # First create a memory with tags
        await client.post(
            "/api/v1/memories",
            json={
                "title": "Test Memory",
                "content": "Test content",
                "type": "short_term",
                "tags": ["test", "api"],
            },
            headers=auth_headers,
        )

        # Get stats
        response = await client.get("/api/v1/stats", headers=auth_headers)
        data = response.json()

        # If there are tags, verify format
        if data["most_used_tags"]:
            first_tag = data["most_used_tags"][0]
            assert isinstance(first_tag, list)
            assert len(first_tag) == 2
            assert isinstance(first_tag[0], str)  # tag name
            assert isinstance(first_tag[1], int)  # count


@pytest.mark.asyncio
async def test_tags_endpoint_uses_correct_method(test_db, auth_headers, memory_dir):
    """
    Verify tags endpoint calls list_all_tags.

    Bug fix: endpoint called manager.list_tags which doesn't exist.
    Should call manager.list_all_tags and extract tag names.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        response = await client.get("/api/v1/tags", headers=auth_headers)

        # Should not raise AttributeError
        assert response.status_code == 200
        data = response.json()

        # Verify response structure
        assert "success" in data
        assert "tags" in data
        assert "total" in data

        # Verify types
        assert isinstance(data["tags"], list)
        assert isinstance(data["total"], int)


@pytest.mark.asyncio
async def test_tags_endpoint_with_memories(test_db, auth_headers, memory_dir):
    """
    Verify tags endpoint returns actual tags from memories.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        # Create memories with tags
        await client.post(
            "/api/v1/memories",
            json={
                "title": "Memory 1",
                "content": "Content 1",
                "type": "short_term",
                "tags": ["python", "testing"],
            },
            headers=auth_headers,
        )

        await client.post(
            "/api/v1/memories",
            json={
                "title": "Memory 2",
                "content": "Content 2",
                "type": "long_term",
                "tags": ["python", "api"],
            },
            headers=auth_headers,
        )

        # Get tags
        response = await client.get("/api/v1/tags", headers=auth_headers)
        data = response.json()

        # Should have tags from both memories
        assert data["total"] >= 2  # At least "python" and one other
        assert "python" in data["tags"]  # Common tag

        # All tags should be strings
        for tag in data["tags"]:
            assert isinstance(tag, str)


@pytest.mark.asyncio
async def test_api_key_validation_with_underscores(test_db):
    """
    Verify API key validation handles underscores in random part.

    Bug fix: split("_") caused keys with underscores to fail.
    Now uses split("_", 2) to properly handle them.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        # Create user and key with underscore in random part
        async with test_db.async_session() as session:
            user = User(
                email="underscore@example.com", plan_tier="pro", whop_user_id="test_underscore"
            )
            session.add(user)
            await session.commit()
            await session.refresh(user)

            # Manually create a key that has underscores in random part
            from whitemagic.api.auth import hash_api_key

            test_key = "wm_prod_test_key_with_underscores"
            key_hash = hash_api_key(test_key)

            api_key = APIKey(
                user_id=user.id,
                key_hash=key_hash,
                key_prefix="wm_prod_test_ke",  # 16 chars
                is_active=True,
            )
            session.add(api_key)
            await session.commit()

        # Try to use the key with underscores
        response = await client.get(
            "/api/v1/stats", headers={"Authorization": f"Bearer {test_key}"}
        )

        # Should authenticate successfully
        assert response.status_code == 200


@pytest.mark.asyncio
async def test_consolidated_endpoint_actual_consolidation(test_db, auth_headers, memory_dir):
    """
    Integration test: Verify consolidate endpoint actually consolidates memories.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        # Create an old short-term memory (simulate by not doing dry run)
        # Note: This is a simplified test - real consolidation requires old memories

        response = await client.post(
            "/api/v1/consolidate",
            json={"dry_run": False, "min_age_days": 0},  # Consolidate everything
            headers=auth_headers,
        )

        # May return 422 if no memories, 500 if manager issue, 200 if success
        assert response.status_code in (200, 422, 500)
        if response.status_code == 200:
            data = response.json()
            # Verify response
            assert data["success"] is True
            assert "archived_count" in data
            assert "promoted_count" in data
            assert data["dry_run"] is False


@pytest.mark.asyncio
async def test_stats_count_consistency(test_db, auth_headers, memory_dir):
    """
    Verify total_count equals short_term_count + long_term_count.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        # Create some memories
        await client.post(
            "/api/v1/memories",
            json={"title": "ST1", "content": "C1", "type": "short_term", "tags": []},
            headers=auth_headers,
        )
        await client.post(
            "/api/v1/memories",
            json={"title": "LT1", "content": "C2", "type": "long_term", "tags": []},
            headers=auth_headers,
        )

        # Get stats
        response = await client.get("/api/v1/stats", headers=auth_headers)
        data = response.json()

        # Verify count consistency
        assert data["total_count"] == data["short_term_count"] + data["long_term_count"]

==> whitemagic-2.2.9.tar.gz: whitemagic-2.2.9/tests/test_api_recent_fixes.py <==
"""
Integration tests for recently fixed API endpoints.

These tests verify the fixes from the third independent review:
1. consolidate endpoint uses consolidate_short_term (not consolidate_memories)
2. stats endpoint builds stats from list_all_memories and list_all_tags
3. tags endpoint uses list_all_tags (not list_tags)
"""

import os
import shutil
import tempfile
from pathlib import Path

import pytest
import pytest_asyncio
from httpx import ASGITransport, AsyncClient

from whitemagic.api.app import app
from whitemagic.api.auth import generate_api_key
from whitemagic.api.database import APIKey, Database, User
from whitemagic.api.dependencies import set_database


@pytest_asyncio.fixture
async def test_db():
    """Create a test database."""
    db = Database("sqlite+aiosqlite:///:memory:")
    await db.create_tables()
    set_database(db)  # Register with dependency injection
    yield db
    await db.close()


@pytest_asyncio.fixture
async def test_user(test_db):
    """Create a test user with API key."""
    async with test_db.async_session() as session:
        # Create user
        user = User(email="test@example.com", plan_tier="pro", whop_user_id="test_user_123")
        session.add(user)
        await session.commit()
        await session.refresh(user)

        # Generate API key
        full_key, key_hash, key_prefix = generate_api_key("prod")

        api_key = APIKey(
            user_id=user.id,
            key_hash=key_hash,
            key_prefix=key_prefix,
            is_active=True,
        )
        session.add(api_key)
        await session.commit()

        yield user, full_key


@pytest_asyncio.fixture
async def auth_headers(test_user):
    """Create authorization headers."""
    _, api_key = test_user
    return {"Authorization": f"Bearer {api_key}"}


@pytest_asyncio.fixture
async def memory_dir():
    """Create a temporary directory for test memories."""
    temp_dir = tempfile.mkdtemp()

    # Create memory structure
    memory_path = Path(temp_dir) / "memory"
    (memory_path / "short_term").mkdir(parents=True, exist_ok=True)
    (memory_path / "long_term").mkdir(parents=True, exist_ok=True)
    (memory_path / "archive").mkdir(parents=True, exist_ok=True)

    # Set environment variable
    old_base = os.environ.get("WM_BASE_PATH")
    os.environ["WM_BASE_PATH"] = temp_dir

    yield temp_dir

    # Cleanup
    if old_base:
        os.environ["WM_BASE_PATH"] = old_base
    else:
        os.environ.pop("WM_BASE_PATH", None)

    shutil.rmtree(temp_dir, ignore_errors=True)


@pytest.mark.asyncio
async def test_consolidate_endpoint_uses_correct_method(test_db, auth_headers, memory_dir):
    """
    Verify consolidate endpoint calls consolidate_short_term.

    This was the primary bug: endpoint called manager.consolidate_memories
    which doesn't exist. Should call manager.consolidate_short_term.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        response = await client.post(
            "/api/v1/consolidate", json={"dry_run": True, "min_age_days": 30}, headers=auth_headers
        )

        # Should not raise AttributeError for missing method
        assert response.status_code == 200
        data = response.json()

        # Verify response structure
        assert "success" in data
        assert "archived_count" in data
        assert "promoted_count" in data
        assert "dry_run" in data
        assert data["dry_run"] is True


@pytest.mark.asyncio
async def test_stats_endpoint_format(test_db, auth_headers, memory_dir):
    """
    Verify stats endpoint returns correct format.

    Bug fix: endpoint called manager.get_stats which doesn't exist.
    Now builds stats from list_all_memories and list_all_tags.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        response = await client.get("/api/v1/stats", headers=auth_headers)

        assert response.status_code == 200
        data = response.json()

        # Verify required fields
        assert "success" in data
        assert "short_term_count" in data
        assert "long_term_count" in data
        assert "total_count" in data
        assert "total_tags" in data
        assert "most_used_tags" in data

        # Verify types
        assert isinstance(data["short_term_count"], int)
        assert isinstance(data["long_term_count"], int)
        assert isinstance(data["total_count"], int)
        assert isinstance(data["total_tags"], int)
        assert isinstance(data["most_used_tags"], list)


@pytest.mark.asyncio
async def test_stats_most_used_tags_format(test_db, auth_headers, memory_dir):
    """
    Verify most_used_tags is properly formatted as list of [tag, count] tuples.

    This ensures the fix properly extracts tag data from list_all_tags response.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        # First create a memory with tags
        await client.post(
            "/api/v1/memories",
            json={
                "title": "Test Memory",
                "content": "Test content",
                "type": "short_term",
                "tags": ["test", "api"],
            },
            headers=auth_headers,
        )

        # Get stats
        response = await client.get("/api/v1/stats", headers=auth_headers)
        data = response.json()

        # If there are tags, verify format
        if data["most_used_tags"]:
            first_tag = data["most_used_tags"][0]
            assert isinstance(first_tag, list)
            assert len(first_tag) == 2
            assert isinstance(first_tag[0], str)  # tag name
            assert isinstance(first_tag[1], int)  # count


@pytest.mark.asyncio
async def test_tags_endpoint_uses_correct_method(test_db, auth_headers, memory_dir):
    """
    Verify tags endpoint calls list_all_tags.

    Bug fix: endpoint called manager.list_tags which doesn't exist.
    Should call manager.list_all_tags and extract tag names.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        response = await client.get("/api/v1/tags", headers=auth_headers)

        # Should not raise AttributeError
        assert response.status_code == 200
        data = response.json()

        # Verify response structure
        assert "success" in data
        assert "tags" in data
        assert "total" in data

        # Verify types
        assert isinstance(data["tags"], list)
        assert isinstance(data["total"], int)


@pytest.mark.asyncio
async def test_tags_endpoint_with_memories(test_db, auth_headers, memory_dir):
    """
    Verify tags endpoint returns actual tags from memories.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        # Create memories with tags
        await client.post(
            "/api/v1/memories",
            json={
                "title": "Memory 1",
                "content": "Content 1",
                "type": "short_term",
                "tags": ["python", "testing"],
            },
            headers=auth_headers,
        )

        await client.post(
            "/api/v1/memories",
            json={
                "title": "Memory 2",
                "content": "Content 2",
                "type": "long_term",
                "tags": ["python", "api"],
            },
            headers=auth_headers,
        )

        # Get tags
        response = await client.get("/api/v1/tags", headers=auth_headers)
        data = response.json()

        # Should have tags from both memories
        assert data["total"] >= 2  # At least "python" and one other
        assert "python" in data["tags"]  # Common tag

        # All tags should be strings
        for tag in data["tags"]:
            assert isinstance(tag, str)


@pytest.mark.asyncio
async def test_api_key_validation_with_underscores(test_db):
    """
    Verify API key validation handles underscores in random part.

    Bug fix: split("_") caused keys with underscores to fail.
    Now uses split("_", 2) to properly handle them.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        # Create user and key with underscore in random part
        async with test_db.async_session() as session:
            user = User(
                email="underscore@example.com", plan_tier="pro", whop_user_id="test_underscore"
            )
            session.add(user)
            await session.commit()
            await session.refresh(user)

            # Manually create a key that has underscores in random part
            from whitemagic.api.auth import hash_api_key

            test_key = "wm_prod_test_key_with_underscores"
            key_hash = hash_api_key(test_key)

            api_key = APIKey(
                user_id=user.id,
                key_hash=key_hash,
                key_prefix="wm_prod_test_ke",  # 16 chars
                is_active=True,
            )
            session.add(api_key)
            await session.commit()

        # Try to use the key with underscores
        response = await client.get(
            "/api/v1/stats", headers={"Authorization": f"Bearer {test_key}"}
        )

        # Should authenticate successfully
        assert response.status_code == 200


@pytest.mark.asyncio
async def test_consolidated_endpoint_actual_consolidation(test_db, auth_headers, memory_dir):
    """
    Integration test: Verify consolidate endpoint actually consolidates memories.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        # Create an old short-term memory (simulate by not doing dry run)
        # Note: This is a simplified test - real consolidation requires old memories

        response = await client.post(
            "/api/v1/consolidate",
            json={"dry_run": False, "min_age_days": 0},  # Consolidate everything
            headers=auth_headers,
        )

        # May return 422 if no memories, 500 if manager issue, 200 if success
        assert response.status_code in (200, 422, 500)
        if response.status_code == 200:
            data = response.json()
            # Verify response
            assert data["success"] is True
            assert "archived_count" in data
            assert "promoted_count" in data
            assert data["dry_run"] is False


@pytest.mark.asyncio
async def test_stats_count_consistency(test_db, auth_headers, memory_dir):
    """
    Verify total_count equals short_term_count + long_term_count.
    """
    async with AsyncClient(transport=ASGITransport(app=app), base_url="http://test") as client:
        # Create some memories
        await client.post(
            "/api/v1/memories",
            json={"title": "ST1", "content": "C1", "type": "short_term", "tags": []},
            headers=auth_headers,
        )
        await client.post(
            "/api/v1/memories",
            json={"title": "LT1", "content": "C2", "type": "long_term", "tags": []},
            headers=auth_headers,
        )

        # Get stats
        response = await client.get("/api/v1/stats", headers=auth_headers)
        data = response.json()

        # Verify count consistency
        assert data["total_count"] == data["short_term_count"] + data["long_term_count"]
